Attribute VB_Name = "basUtils"
'Some functions, used by yvErrorHandler
'You may put them into yvErrorHandler module and make them private...
Option Explicit
Option Compare Text
Const conModuleName = "basUtils"

Public Const gstrSEP_DIR As String = "\"                         ' Directory separator character
Public Const gstrSEP_URLDIR As String = "/"                      ' Separator for dividing directories in URL addresses.

'-----------------------------------------------------------
' Function: AddDirSep
' Add a trailing directory path separator (back slash) to the
' end of a pathname unless one already exists
'
' IN/OUT: [strPathName] - path to add separator to
'-----------------------------------------------------------
'
Public Function AddDirSep(strPathName As String, Optional ByVal blnURL As Boolean = False) As String
Dim str1 As String
  str1 = strPathName
  If str1 = "" Then GoTo ExitProc
  If Right(Trim(str1), Len(gstrSEP_URLDIR)) <> gstrSEP_URLDIR And _
    Right(Trim(str1), Len(gstrSEP_DIR)) <> gstrSEP_DIR Then
     If blnURL Then
        str1 = RTrim$(str1) & gstrSEP_URLDIR
     Else
        str1 = RTrim$(str1) & gstrSEP_DIR
     End If
  End If

ExitProc:
  strPathName = str1
  AddDirSep = str1
  Exit Function
End Function

' Write string to the file (optionally append to the existing file)
' Return: file size
Public Function String2File(ByVal str1 As String, ByVal strFileName As String, Optional blnAppend As Boolean = False, Optional ByVal blnErrorReport As Boolean = True) As Long
Dim blnOk As Boolean
Const conModuleSubName = conModuleName & ":" & "String2File function"
Dim hFile As Long
Dim lngFilePointer0 As Long
Dim lngFileLength As Long
  On Error GoTo ErrProc
  
  If Not blnAppend Then
    hFile = FreeFile
    Open strFileName For Output As #hFile
    Close #hFile
  End If
  
  hFile = FreeFile
  Open strFileName For Binary Access Write As #hFile
  lngFilePointer0 = LOF(hFile)
  Put #hFile, lngFilePointer0 + 1, str1
  lngFileLength = LOF(hFile)
  Close #hFile

  blnOk = True
ExitProc:
  On Error Resume Next
  Close #hFile
  If blnOk Then
    String2File = lngFileLength
  Else
    String2File = 0
    If Not blnAppend Then
      MyKill strFileName
    End If
  End If
  Exit Function
ErrProc:
  If blnErrorReport Then
    'Don't call ShowError functions if this (String2File) function is called by
    'some error handling function
    Call ShowError(conModuleSubName)
  End If
  Resume ExitProc
End Function

'Convert anything into number
'Date is converted into double
'If var1 is not a number, then return Default
Public Function CNum(var1 As Variant, Optional ByVal Default As Variant = 0#) As Variant
  On Error Resume Next
  If IsNull(var1) Then GoTo DefaultProc
  If IsEmpty(var1) Then GoTo DefaultProc

  If IsNumeric(var1) Then
    Err.Clear
    CNum = var1
    If Err.Number = 0 Then Exit Function
  End If
  If IsDate(var1) Then
    Err.Clear
    CNum = CDbl(var1)
    If Err.Number = 0 Then Exit Function
  End If

DefaultProc:
  CNum = Default
  Exit Function
End Function

Public Function F0(ByVal i1 As Variant) As String
  F0 = Format(i1, "0")
End Function

'MS Access emulation
Public Function Nz(ByVal v1, Optional ByVal ValueIfNull = Empty) As Variant
Dim blnError As Boolean
  On Error GoTo ErrProc
  If IsNull(v1) Then
    Nz = ValueIfNull
  Else
    Nz = v1
  End If
  If blnError Then
    blnError = False
    Nz = ValueIfNull
    If blnError Then
      Nz = Empty
    End If
  End If
    
ExitProc:
  Err.Clear
  Exit Function
  
ErrProc:
  blnError = True
  Err.Clear
  Resume Next
End Function

' encodes date according to the value and scheme
' WTN8601 example:  "1999-03-26T12:46+03:00"
Public Function FDateISO(ByVal dtmValue As Variant) As String
Const conModuleSubName = conModuleName & ":" & "FDateISO function"

Dim dtm1 As Date
Dim str1 As String

Dim strOut As String
  On Error GoTo ErrProc
  
  If Not IsDate(dtmValue) Then GoTo ExitProc
  dtm1 = CVDate(dtmValue)
  
  strOut = Format(dtm1, "yyyy\-mm\-dd")
  
  If Int(dtm1) = dtm1 Then GoTo ExitProc
  
  str1 = Format(dtm1, "Hh\:Nn\:Ss")
  
  strOut = strOut & "T" & str1 'this is right, if this PC is in GMT time zone
  
ExitProc:
  On Error Resume Next
  FDateISO = strOut
  Exit Function

ErrProc:
  Call ShowError(conModuleSubName, "Value: " & Nz(dtmValue, ""))
  Resume ExitProc
End Function

'class ID generator
Public Function GetNextClassDebugID() As Long
Static lClassDebugID As Long
  If lClassDebugID = 0 Then
    'very random initial number in order to get unique instance ID
    Randomize
    lClassDebugID = (Int(Rnd() * 10000) + 1) * 1000
  End If
  lClassDebugID = lClassDebugID + 1
  GetNextClassDebugID = lClassDebugID
End Function

'idle for nSeconds
Public Sub MySleep(nSeconds)
Dim v1 As Variant
  If Not IsNumeric(nSeconds) Then Exit Sub
  v1 = Now
  Do Until DateDiff("s", v1, Now, vbSunday) > nSeconds
    DoEvents
  Loop
End Sub

Public Sub MyKill(ByVal PathName As String)
  On Error Resume Next
  Kill PathName
End Sub

'This function is for MS Access compatibility only
Public Sub Hourglass(ByVal blnOn As Boolean)
Const conModuleSubName = conModuleName & ":" & "Hourglass sub"
  On Error GoTo ErrProc

ExitProc:
  On Error Resume Next
  Exit Sub

ErrProc:
  ShowError conModuleSubName
  Resume ExitProc
End Sub

Public Function FullVersionNumber() As String
Dim str1 As String
  str1 = App.Major & "." & App.Minor & "." & App.Revision
  #If gconIsAlfa <> 0 Then
    str1 = str1 & "a"
  #End If
  #If gconIsBeta <> 0 Then
    str1 = str1 & "b"
  #End If
  FullVersionNumber = str1
End Function

