Attribute VB_Name = "yvErrorHandler"
' Main Error Handling module
' (c)2000-2003 Yuri Volkov, programmer@iname.com
' version 1.12 (2003-10-06)
'-------------------------------------------------------------------------------
' yvErrorHandler features:
' - Universal error handling and logging for all types of errors/warnings.
'   Just insert call to one of ShowError... functions
'   in your (error handling) code
' - Tries to find all errors:
'   it may report more, than one error per call
'   to help you find 'real' cause of problem
' - Knows, that there are "custom errors"
'   (If Err.Number > vbObjectError And Err.Number < vbObjectError + 65536 )
' - Formatting errors for Plain text, HTML and XML clients
'     (see clientContentType function)
' - Trace helper fuctions: TraceStart, TraceEnd and MyTrace
' - Works well for unattended applications (e.g. server components)
' - The same source code for MS Access 97 ... 2002 and VB5 ... 6
' - Adapted for convertion to the VB.NET
' - Freeware
'-------------------------------------------------------------------------------
' This module has quite a lot of external dependencies.
' So it is distributed as a part of complete VB application,
'   that you may use as a template.
'
'-------------------------------------------------------------------------------
' Revision history
' 2003-10-06 - Module formatting, additional source code comments
' 2003-06-06 - Fixed error creating xmlError (namespace was missed)
' 2003-04-25 - Got rid of "GoSub" statement... (preparing to .NET)
'              Globals renamed to AppGlobals
' 2003-04-17 - AppendClientString function: parameter 'LogIt' added
' 2003-04-09 - clientContentType function added,
'              AppendClientString adds xml strings for xml client...
' 06.11.2002 - _ByVal_ LogBuffer in MyLogEvent Sub
' 11.12.2001 - Globals.conn1... renamed to Globals.cnn1
'             (means Connection, but "con" is the prefix for Constant !)
' 19.11.2001 - AppendClientString function added
' 04.09.2001 - clientCrLf changed for text/xml
' 30.08.2001 - new XML type library: MSXML2
' 23.08.2001 - checking Err object moved before other error objects
' 06.06.2001 - fixed additional error reports for custom errors with codes <=100
Option Explicit
Option Base 0
Private Const conModuleName = "yvErrorHandler"

'Set 1 for the libraries, that are referenced in your project
#Const includeADO = 1
#Const includeDAO = 0
#Const includeXML = 1

#If includeXML Then
'Of cause, you may change namespace as you wish
'For NGOSS application
'Global Const gconXMLErrorNs As String = "http://ngo.org.ru/ngoss/messages/1.0/"
'Global Const gconXMLErrorPrefix As String = "ngom"
'For other YV applications
Global Const gconXMLErrorNs As String = "http://ngo.org.ru/ngoss/errors/1.0/"
Global Const gconXMLErrorPrefix As String = "err"
#End If

Global Const ERROR_SUCCESS As Long = 0
Global Const ERROR_INVALID_FUNCTION As Long = 1
Global Const ERROR_FILE_NOT_FOUND As Long = 2
Global Const ERROR_PATH_NOT_FOUND As Long = 3
Global Const ERROR_TOO_MANY_OPEN_FILES As Long = 4
Global Const ERROR_ACCESS_DENIED As Long = 5
Global Const ERROR_INVALID_HANDLE As Long = 6
Global Const ERROR_INVALID_PARAMETER As Long = 87
Global Const ERROR_PIPE_LISTENING As Long = 536
Global Const ERROR_IO_PENDING As Long = 997
' to do other error codes (maybe ...)

Public Declare Function GetLastError Lib "kernel32" () As Long

'for ShowErrorSQL function
Private nErrorCodes As Long
Private aErrorCodes() As Long
Private strErr As String
Private strErrText As String
Private strAdd As String
Private strAddText As String

'The yvErrorHelper 'knows' three Content Types:
' 1. "text/html"
'    (or 'aliases': "application/x-www-form-urlencoded", "multipart/form-data")
' 2. "text/xml"
' 3. "text" and all other content types are regarded as 'Plain Text'
'Trying to use AppGlobals.ContentType
Private Function clientContentType() As String
Static strContentType As String
  If strContentType = "" Then
    If Not AppGlobals Is Nothing Then
      strContentType = AppGlobals.ContentType
    End If
  End If
  If strContentType = "" Then
    strContentType = "text"
  Else
    clientContentType = strContentType
  End If
End Function

'Show message only
Public Function ShowErrorMessage( _
  ByVal strModuleSubName As String _
  , ByVal strMessage As String _
  ) As String
  
  ShowErrorMessage = ShowErrorSql(strModuleSubName, "", strMessage, True, "")
End Function

'Show message (error description in this case)
'  plus 'parse' error object, passed as a parameter
Public Function ShowError( _
  ByVal strModuleSubName As String _
  , Optional ByVal strAdditionalInfo As String = "" _
  , Optional ByVal blnWait As Boolean = True _
  , Optional ByVal varErr As Variant = Empty _
  ) As String
  
  ShowError = ShowErrorSql(strModuleSubName, "", strAdditionalInfo, blnWait, varErr)
End Function

'Main Error Handling function, it has largest number of parameters
'(note that VB doesn't support function overloading)
'  plus additional information about Sql Statement
'If blnWait and Not gblnUnattendedApp: opens Dialog window (MsgBox)
'  with error description
'
'Returns: Pain Text error description
Public Function ShowErrorSql( _
    ByVal strModuleSubName As String _
  , ByVal strSql As String _
  , Optional ByVal strAdditionalInfo As String = "" _
  , Optional ByVal blnWait As Boolean = True _
  , Optional ByVal varErr As Variant = Empty _
  ) As String
Const conErrorCodes As Long = 20
Dim str1 As String
Dim strWin As String
Dim bln1 As Boolean

' Error object
Dim Err1 As Variant
Dim lngError As Long

'error code > vbObjectError And  < vbObjectError + 65536
Dim blnCustomError As Boolean
'error code > vbObjectError And  = vbObjectError + 100
Dim blnCustomError100 As Boolean

  'clear module level variables
  nErrorCodes = 0
  ReDim aErrorCodes(conErrorCodes - 1)
  strErr = ""
  strErrText = ""
  strAdd = ""
  strAddText = ""

  If strAdditionalInfo <> "" Then
    Call AppendClientString(strAdd, strAdditionalInfo)
    If strAddText <> "" Then strAddText = strAddText & vbCrLf
    strAddText = strAddText & strAdditionalInfo
  End If
  
  If VarType(varErr) = vbNull Or VarType(varErr) = vbEmpty Then
    lngError = GetLastError
    If lngError <> 0 Then
      Err1 = lngError
      str1 = "Windows reported error:" & WinErrorDescribed(lngError)
      Call AppendClientString(strAdd, str1)
      If strAddText <> "" Then strAddText = strAddText & vbCrLf
      strAddText = strAddText & str1
      Call ShowErr1Proc(strModuleSubName, strSql, Err1)
    End If
    
    If Err.Number > vbObjectError And Err.Number < vbObjectError + 65536 Then
      blnCustomError = True
      If Err.Number <= vbObjectError + 100 Then
        blnCustomError100 = True
      End If
      Set Err1 = Err
      Call ShowErr1Proc(strModuleSubName, strSql, Err1)
    End If
    If Not blnCustomError100 Then
      If Err.Number <> 0 Then
        Set Err1 = Err
        Call ShowErr1Proc(strModuleSubName, strSql, Err1)
      End If
      
      'Don't show other errors, if these errors are "custom": from #1 to #100  !!!!!!
#If includeADO Then
      If Not (AppGlobals Is Nothing) Then
        If Not AppGlobals.cnn1IsNothing Then
          For Each Err1 In AppGlobals.cnn1.Errors
            Call ShowErr1Proc(strModuleSubName, strSql, Err1)
          Next Err1
        End If
      End If
#End If
#If includeDAO Then
      For Each Err1 In DBEngine.Errors
        Call ShowErr1Proc(strModuleSubName, strSql, Err1)
      Next Err1
#End If
    End If
    
    If strErrText = "" Then
      Err1 = Empty
      Call ShowErr1Proc(strModuleSubName, strSql, Err1)
    End If
  ElseIf IsObject(varErr) Then
    Set Err1 = varErr
    Call ShowErr1Proc(strModuleSubName, strSql, Err1)
  Else
    Err1 = varErr
    Call ShowErr1Proc(strModuleSubName, strSql, Err1)
  End If
  
  Call MyLogEvent(strErrText)
  
  If Not gblnUnattendedApp Then Call Hourglass(False)
  
  If gblnUnattendedApp Or (Not blnWait) Then
    If Not AppGlobals Is Nothing Then
      AppGlobals.ErrorFormatted = AppendClientString(AppGlobals.ErrorFormatted, strErr)
      Let AppGlobals.ErrorFormatted_last = strErr
    End If
  Else
    MsgBox strErrText & vbCrLf & vbCrLf & "Current time: " & Now, vbOKOnly, gstrTitle
  End If
  
ExitProc:
  ShowErrorSql = strErr
  Erase aErrorCodes
  Exit Function
End Function

'Parse exactly one error (Err1 object)
Private Function ShowErr1Proc( _
    ByRef strModuleSubName As String _
  , ByRef strSql As String _
  , ByRef Err1 As Variant _
  ) As Boolean
Dim lng1 As Long
Dim ErrorNumber As Long
Dim blnNewError As Boolean
  
  If IsObject(Err1) Then
    If (TypeOf Err1 Is VBA.ErrObject) Then
      ErrorNumber = Err1.Number
#If includeXML Then
    ElseIf (TypeOf Err1 Is MSXML2.IXMLDOMParseError) Then
      ErrorNumber = Err1.errorCode
#End If
#If includeADO Then
    ElseIf (TypeOf Err1 Is ADODB.Error) Then
      ErrorNumber = Err1.Number
#End If
#If includeDAO Then
    ElseIf (TypeOf Err1 Is DAO.Error) Then
      ErrorNumber = Err1.Number
#End If
    Else
      ErrorNumber = vbObjectError + nErrorCodes
    End If
  ElseIf IsNumeric(Err1) Then
    ErrorNumber = CNum(Err1)
  Else
    ErrorNumber = vbObjectError + 1
  End If
  
  ' Trying not to show one error more than once
  ' find current error number: ErrorNumber
  blnNewError = True
  For lng1 = 0 To nErrorCodes - 1
    If aErrorCodes(lng1) = ErrorNumber Then
      blnNewError = False
      Exit For
    End If
  Next lng1
  
  If blnNewError Then
    If nErrorCodes < UBound(aErrorCodes) Then
      nErrorCodes = nErrorCodes + 1
      aErrorCodes(nErrorCodes - 1) = ErrorNumber
    End If
    If strErrText <> "" Then
      'append additional info only once!
      strAdd = ""
      strAddText = ""
      strSql = ""
      strErr = strErr & clientCrLf & clientCrLf
      strErrText = strErrText & vbCrLf & vbCrLf
    End If
  End If
  
  If blnNewError Then
    strErrText = strErrText & Err2TextSql(Err1, strModuleSubName, strSql, strAddText, vbCrLf)
    Select Case clientContentType()
    Case "text/html", "application/x-www-form-urlencoded", "multipart/form-data"
      strErr = strErr & Err2TextSql(Err1, strModuleSubName, strSql, strAddText, clientCrLf)
    Case "text/xml"
      strErr = strErr & Err2XMLSql(Err1, strModuleSubName, strSql, strAddText, clientCrLf)
    Case Else
      strErr = strErrText
    End Select
  End If
  Exit Function
End Function

' format different errors as Pain Text or HTML
Private Function Err2TextSql(ByRef varError As Variant, ByVal strModuleSubName As String, ByVal strSql As String, ByVal strAdditionalInfo As String, ByVal strCrLf As String) As String
Dim str1 As String
Dim strDescription As String
Dim lngError As Long
  str1 = ""
  If IsObject(varError) Then
    If (TypeOf varError Is VBA.ErrObject) Then
      lngError = varError.Number
      If lngError > vbObjectError And lngError < vbObjectError + 65536 Then
        lngError = lngError - vbObjectError
        str1 = str1 & "error object: custom object" & strCrLf
      Else
        str1 = str1 & "error object: VBA.Err" & strCrLf
      End If
      str1 = str1 & "error number: " & F0(lngError) & strCrLf
      str1 = str1 & "source: " & varError.Source & strCrLf
      strDescription = "description: " & varError.Description
#If includeXML Then
    ElseIf (TypeOf varError Is MSXML2.IXMLDOMParseError) Then
      lngError = varError.errorCode
      str1 = str1 & "error object: MSXML2.IXMLDOMParseError" & strCrLf
      str1 = str1 & "error code: " & F0(lngError) & strCrLf _
                  & "reason: " & varError.reason & strCrLf _
       & "file position: " & varError.filepos & strCrLf _
       & "line: " & varError.Line & strCrLf _
       & "line position: " & varError.linepos & strCrLf _
       & "src Text: " & varError.srcText & strCrLf _
       & "url: " & varError.url & strCrLf
#End If
#If includeADO Then
    ElseIf (TypeOf varError Is ADODB.Error) Then
      lngError = varError.Number
      str1 = str1 & "error object: ADODB.Error" & strCrLf
      str1 = str1 & "error code: " & F0(lngError) & strCrLf _
       & "Description: " & varError.Description & strCrLf _
       & "native error: " & varError.NativeError & strCrLf _
       & "SQL State: " & varError.SQLState & strCrLf _
       & "src Text: " & varError.Source & strCrLf
#End If
#If includeDAO Then
    ElseIf (TypeOf varError Is DAO.Error) Then
      lngError = varError.Number
      str1 = str1 & "error object: DAO.Error" & strCrLf
      str1 = str1 & "error number: " & F0(varError.Number) & strCrLf _
                  & "source: " & varError.Source & strCrLf
      strDescription = "description: " & varError.Description
#End If
    Else
      str1 = str1 & "error object: unknown" & strCrLf
    End If
  ElseIf IsNumeric(varError) Then
    str1 = str1 & "error code: " & F0(CNum(varError)) & strCrLf
  Else
    str1 = str1 & "reason: " & Nz(varError, "") & strCrLf
  End If
  If strModuleSubName <> "" Then
    str1 = str1 & "Code module: " & strModuleSubName & strCrLf
  End If
  If strSql <> "" Then
    str1 = str1 & "Last SQL: '" & strSql & "'" & strCrLf
  End If
  If strAdditionalInfo <> "" Then
    str1 = str1 & "Notes: " & strCrLf & Trim(strAdditionalInfo) & strCrLf
  End If
  If strDescription <> "" Then
    str1 = str1 & strDescription & strCrLf
  End If
  Err2TextSql = str1
End Function

' format different errors as XML
Private Function Err2XMLSql(ByRef varError As Variant, ByVal strModuleSubName As String, ByVal strSql As String, ByVal strAdditionalInfo As String, ByVal strCrLf As String) As String
Dim blnOk As Boolean
Dim str1 As String
Dim aErr(1 To 8) As Variant
Dim aName(1 To 8) As String

Dim strErrorObject As String
Dim lngError As Long

Dim lng1 As Long

Dim strOut As String

#If includeXML Then

Dim node1 As MSXML2.IXMLDOMNode
Dim node2 As MSXML2.IXMLDOMNode
Dim obj1 As Object
  
  If AppGlobals Is Nothing Then GoTo Exit0Proc
  If AppGlobals.xmlError Is Nothing Then
    Set AppGlobals.xmlError = New MSXML2.FreeThreadedDOMDocument40
    AppGlobals.xmlError.async = False
    Call AppGlobals.xmlError.setProperty("SelectionNamespaces", _
    "xmlns:" & gconXMLErrorPrefix & "='" & gconXMLErrorNs & "'")
    AppGlobals.xmlError.loadXML ( _
    "<?xml version='1.0' encoding='windows-1251' ?>" _
    & "<document xmlns:" & gconXMLErrorPrefix & "='" & gconXMLErrorNs & "' />")
  End If
  
  If IsObject(varError) Then
    If (TypeOf varError Is VBA.ErrObject) Then
      strErrorObject = "VBA.Err"
      lngError = varError.Number
      
      lng1 = lng1 + 1
      aErr(lng1) = varError.Description
      aName(lng1) = "Description"
      lng1 = lng1 + 1
      aErr(lng1) = varError.Source
      aName(lng1) = "Source"
      
      If lngError > vbObjectError And lngError < vbObjectError + 65536 Then
        strErrorObject = "custom object"
        lngError = lngError - vbObjectError
      End If
#If includeXML Then
    ElseIf (TypeOf varError Is MSXML2.IXMLDOMParseError) Then
      strErrorObject = "MSXML2.IXMLDOMParseError"
      lngError = varError.errorCode
      
      lng1 = lng1 + 1
      aErr(lng1) = varError.filepos
      aName(lng1) = "filepos"
      lng1 = lng1 + 1
      aErr(lng1) = varError.Line
      aName(lng1) = "line"
      lng1 = lng1 + 1
      aErr(lng1) = varError.linepos
      aName(lng1) = "linepos"
      lng1 = lng1 + 1
      aErr(lng1) = varError.reason
      aName(lng1) = "reason"
      lng1 = lng1 + 1
      aErr(lng1) = varError.srcText
      aName(lng1) = "srcText"
      lng1 = lng1 + 1
      aErr(lng1) = varError.url
      aName(lng1) = "url"
      
#End If
#If includeADO Then
    ElseIf (TypeOf varError Is ADODB.Error) Then
      strErrorObject = "ADODB.Error"
      lngError = varError.Number

      lng1 = lng1 + 1
      aErr(lng1) = varError.Description
      aName(lng1) = "reason"
      lng1 = lng1 + 1
      aErr(lng1) = varError.NativeError
      aName(lng1) = "NativeError"
      lng1 = lng1 + 1
      aErr(lng1) = varError.SQLState
      aName(lng1) = "SQLState"
      lng1 = lng1 + 1
      aErr(lng1) = varError.Source
      aName(lng1) = "Source"

#End If
#If includeDAO Then
    ElseIf (TypeOf varError Is DAO.Error) Then
      lngError = varError.Number
      strErrorObject = "ADODB.Error"

      lng1 = lng1 + 1
      aErr(lng1) = varError.Description
      aName(lng1) = "Description"
      lng1 = lng1 + 1
      aErr(lng1) = varError.NativeError
      aName(lng1) = "NativeError"
      lng1 = lng1 + 1
      aErr(lng1) = varError.SQLState
      aName(lng1) = "SQLState"
      lng1 = lng1 + 1
      aErr(lng1) = varError.Source
      aName(lng1) = "Source"

#End If
    Else
      strErrorObject = "unknown"
      lngError = vbObjectError + 1
    End If
  ElseIf IsNumeric(varError) Then
    lngError = CNum(varError)
  Else
    ' some unknown error
    lngError = vbObjectError + 1
    
    lng1 = lng1 + 1
    aErr(lng1) = Nz(varError, "")
    aName(lng1) = "Description"
  End If
  
  On Error GoTo ErrProc
  
  Set node1 = AppGlobals.xmlError.selectSingleNode("document")
  Set node2 = AppGlobals.xmlError.createNode(NODE_ELEMENT, _
   gconXMLErrorPrefix & ":error", gconXMLErrorNs)
  node1.appendChild node2
  
  Call AddErrorNode(node2, "date", FDateISO(Now))
  Call AddErrorNode(node2, "codeModule", strModuleSubName)
  Call AddErrorNode(node2, "notes", strAdditionalInfo)
  Call AddErrorNode(node2, "sql", strSql)
  
  Call AddErrorNode(node2, "object", strErrorObject)
  Call AddErrorNode(node2, "errorCode", F0(lngError))
  
  For lng1 = 1 To 8
    If aName(lng1) <> "" Then
      Call AddErrorNode(node2, aName(lng1), aErr(lng1))
    End If
  Next lng1
  
  strOut = node2.xml & vbCrLf
  
#End If

Exit0Proc:
  If strOut = "" Then
    'XML was not used...
    strOut = Err2TextSql(varError, strModuleSubName, strSql, strAdditionalInfo, strCrLf)
  End If
  blnOk = True
ExitProc:
  On Error Resume Next
  
#If includeXML Then
  Set node1 = Nothing
  Set node2 = Nothing
#End If

  Err2XMLSql = strOut
  Exit Function

ErrProc:
  blnOk = False
  Resume ExitProc
End Function

#If includeXML Then
'add node if value is not empty
Private Function AddErrorNode(ByRef nodeParent As MSXML2.IXMLDOMNode, ByVal strnodeName As String, ByVal nodeValue As Variant) As Boolean
Dim node3 As MSXML2.IXMLDOMNode
Dim node4 As MSXML2.IXMLDOMAttribute
  If strnodeName <> "" And nodeValue <> "" Then
    Set node3 = AppGlobals.xmlError.createNode(NODE_ELEMENT, _
      gconXMLErrorPrefix & ":Property", gconXMLErrorNs)
    nodeParent.appendChild node3
    node3.Text = nodeValue
    
    Set node4 = AppGlobals.xmlError.createNode(NODE_ATTRIBUTE, _
      gconXMLErrorPrefix & ":PropertyName", gconXMLErrorNs)
    node3.Attributes.setNamedItem node4
    node4.Text = strnodeName
    Set node3 = Nothing
    Set node4 = Nothing
  End If
  AddErrorNode = True
End Function
#End If

'CrLf, that is taking clientContentType() into account
'This function helps format user's output
Public Function clientCrLf() As String
Static strCrLf As String
  If strCrLf = "" Then
    Select Case clientContentType()
    Case "text/html", "application/x-www-form-urlencoded", "multipart/form-data"
      strCrLf = "<br>" & vbCrLf
    Case "text/xml"
      strCrLf = "<br/>" & vbCrLf
    Case Else
      strCrLf = vbCrLf
    End Select
  End If
  clientCrLf = strCrLf
End Function

'Append string to the message for the User, taking clientContentType() into account
Public Function AppendClientString(ByRef strMessage As String, ByVal strNewLine As String, Optional ByVal LogIt As Boolean = False) As String
Dim strOut As String
  strOut = strMessage
  If strNewLine <> "" Then
    If LogIt Then
      Call MyLogEvent(strNewLine)
    End If
    Select Case clientContentType()
    Case "text/xml"
      'simplified XML test yet
      If InStr(1, strNewLine, "<", vbBinaryCompare) = 1 Then
        'this is valid element
        strOut = strOut & strNewLine
      Else
        strOut = strOut & "<ClientString>" & strNewLine & "</ClientString>" & vbCrLf
      End If
    Case Else
      If strOut <> "" Then strOut = strOut & clientCrLf
      strOut = strOut & strNewLine
    End Select
  End If
  AppendClientString = strOut
  strMessage = strOut
End Function

'Trace helper fuctions: TraceStart, TraceEnd and MyTrace
Public Sub MyTrace(ByVal strModuleSubName As String, ByVal strEvent As String, ByVal strMessage As String)
Dim str1 As String
  If Not gblnDebug Then Exit Sub
  str1 = "Trace module: '" & strModuleSubName & "'"
  If strEvent <> "" Then
    str1 = str1 & " " & strEvent
  End If
  If strMessage <> "" Then
    str1 = str1 & "; " & strMessage
  End If
  Call MyLogEvent(str1)
End Sub

Public Sub TraceStart(ByVal strModuleSubName As String, Optional ByVal strMessage As String = "")
  If Not gblnDebug Then Exit Sub
  Call MyTrace(strModuleSubName, "started", strMessage)
End Sub

Public Sub TraceEnd(ByVal strModuleSubName As String, Optional ByVal strMessage As String = "")
  If Not gblnDebug Then Exit Sub
  Call MyTrace(strModuleSubName, "ended", strMessage)
End Sub

'Descriptions for some Windows dll errors
Public Function WinErrorDescribed(lngError As Long) As String
Dim strErr As String
Const strSep1 As String = ": "

  Select Case lngError
    Case ERROR_SUCCESS
      strErr = "The operation completed successfully"
    Case ERROR_INVALID_FUNCTION
      strErr = "ERROR_INVALID_FUNCTION"
    Case ERROR_FILE_NOT_FOUND
      strErr = "ERROR_FILE_NOT_FOUND"
    Case ERROR_PATH_NOT_FOUND
      strErr = "ERROR_PATH_NOT_FOUND"
    Case ERROR_TOO_MANY_OPEN_FILES
      strErr = "ERROR_TOO_MANY_OPEN_FILES"
    Case ERROR_ACCESS_DENIED
      strErr = "ERROR_ACCESS_DENIED"
    Case ERROR_INVALID_HANDLE
      strErr = "ERROR_INVALID_HANDLE"
    Case ERROR_IO_PENDING
      strErr = "ERROR_IO_PENDING" & strSep1 & _
        "Overlapped I/O operation is in progress."
    Case ERROR_INVALID_PARAMETER
      strErr = "ERROR_INVALID_PARAMETER" & strSep1 & _
        "The parameter is incorrect"
    Case ERROR_PIPE_LISTENING
      strErr = "ERROR_PIPE_LISTENING" & strSep1 & _
        "Waiting for a process to open the other end of the pipe"
    Case Else
      strErr = "Unknown error " & lngError
  End Select
  WinErrorDescribed = strErr
End Function

'Write string to the log file (and, optionally, to IDE Debug Window)
'EventType is not used yet
Public Sub MyLogEvent(ByVal LogBuffer As String, Optional EventType As Long = 0)
Static strLogFileDir As String      'in order to eliminate another error
Dim strFileName As String
  If strLogFileDir = "" Then
    If gblnUnattendedApp Then
      strLogFileDir = AddDirSep(Environ("SYSTEMROOT")) & "system32\LogFiles\" _
        & gstrApp_EXEName & "\"
    Else
      strLogFileDir = gstrDirWorking
    End If
  End If
  
  'Different log file name for each day
  strFileName = "ex" & Format(Date, "yymmdd") & ".log"
  
  If String2File(vbCrLf & Format(Time(), "hh:nn:ss") & " " _
    & LogBuffer, _
    strLogFileDir & strFileName, True, False) = 0 Then
    
    'Try to create log file in the Windows TEMP folder
    Call String2File(vbCrLf & Time() & " " _
      & "couldn't write to the log file: '" & strFileName & "'" _
      & LogBuffer, _
      AddDirSep(Environ("TEMP")) & strFileName, True, False)
  End If
  
  If gblnDebug Then
    Debug.Print Time & " " & LogBuffer
  End If
  If Not gblnUnattendedApp Then
    DoEvents
  End If
End Sub

